@@ -3,26 +3,46 @@ import h5py
 import numpy as np
 import os
 
+from .models import Selection
+
 try:
     import hdf5plugin  # noqa: F401
 except ImportError:
     pass
-from .utils import attr_metadata, get_entity_from_file, parse_slice, sorted_dict
+from .utils import (
+    attr_metadata,
+    get_array_stats,
+    get_entity_from_file,
+    parse_slice,
+    sorted_dict,
+)
 
 
 class EntityContent:
+    """Base content for an entity."""
+
     type = "other"
 
     def __init__(self, path: str):
         self._path = path
 
     def metadata(self) -> Dict[str, str]:
+        """Entity metadata
+
+        :returns: {"name": str, "type": str}
+        """
         return {"name": self.name, "type": self.type}
 
     @property
     def name(self) -> str:
+        """Entity name. Last member of the path."""
         return self._path.split("/")[-1]
 
+    @property
+    def path(self) -> str:
+        """Path in the file."""
+        return self._path
+
 
 class ExternalLinkContent(EntityContent):
     type = "external_link"
@@ -33,12 +53,26 @@ class ExternalLinkContent(EntityContent):
         self._target_path = link.path
 
     def metadata(self, depth=None):
+        """External link metadata
+
+        :returns: {"name": str, "target_file": str, "target_path": str, "type": str}
+        """
         return sorted_dict(
             ("target_file", self._target_file),
             ("target_path", self._target_path),
             *super().metadata().items(),
         )
 
+    @property
+    def target_file(self) -> str:
+        """The target file of the link"""
+        return self._target_file
+
+    @property
+    def target_path(self) -> str:
+        """The target path of the link (in the target file)"""
+        return self._target_path
+
 
 class SoftLinkContent(EntityContent):
     type = "soft_link"
@@ -46,28 +80,44 @@ class SoftLinkContent(EntityContent):
     def __init__(self, path: str, link: h5py.SoftLink) -> None:
         super().__init__(path)
         self._target_path = link.path
+        """ The target path of the link """
 
     def metadata(self, depth=None):
+        """
+        :returns: {"name": str, "target_path": str, "type": str}
+        """
         return sorted_dict(
             ("target_path", self._target_path), *super().metadata().items()
         )
 
+    @property
+    def target_path(self) -> str:
+        """The target path of the link"""
+        return self._target_path
+
 
 T = TypeVar("T", h5py.Dataset, h5py.Datatype, h5py.Group)
 
 
 class ResolvedEntityContent(EntityContent, Generic[T]):
+    """Content for a link that can be resolved into a h5py entity"""
+
     def __init__(self, path: str, h5py_entity: T):
         super().__init__(path)
         self._h5py_entity = h5py_entity
+        """ Resolved h5py entity """
 
     def attributes(self, attr_keys: Sequence[str] = None):
+        """Attributes of the h5py entity. Can be filtered by keys."""
         if attr_keys is None:
             return dict((*self._h5py_entity.attrs.items(),))
 
         return dict((key, self._h5py_entity.attrs[key]) for key in attr_keys)
 
     def metadata(self, depth=None):
+        """
+        :returns: {"attributes": AttributeMetadata, "name": str, "type": str}
+        """
         attribute_names = sorted(self._h5py_entity.attrs.keys())
         return sorted_dict(
             (
@@ -85,50 +135,38 @@ class DatasetContent(ResolvedEntityContent[h5py.Dataset]):
     type = "dataset"
 
     def metadata(self, depth=None):
+        """
+        :returns: {"attributes": AttributeMetadata, "dtype": str, "shape": tuple, "name": str, "type": str}
+        """
         return sorted_dict(
             ("dtype", self._h5py_entity.dtype.str),
             ("shape", self._h5py_entity.shape),
             *super().metadata().items(),
         )
 
-    def data(self, selection: str = None):
+    def data(self, selection: Selection = None):
+        """Dataset data. Supports slicing through selection."""
         if selection is None:
             return self._h5py_entity[()]
 
-        parsed_slice = parse_slice(self._h5py_entity, selection)
+        if isinstance(selection, str):
+            return self._h5py_entity[parse_slice(self._h5py_entity, selection)]
 
-        return self._h5py_entity[parsed_slice]
+        return self._h5py_entity[selection]
 
-    def data_stats(self, selection: str = None) -> Dict[str, Union[float, int, None]]:
+    def data_stats(
+        self, selection: Selection = None
+    ) -> Dict[str, Union[float, int, None]]:
+        """Statistics on the data. Providing a selection will compute stats only on the selected slice.
+
+        :param selection: NumPy-like indexing to define a selection as a slice
+        :returns: {"strict_positive_min": number | None, "positive_min": number | None, "min": number | None, "max": number | None, "mean": number | None, "std": number | None}
+        """
         data = self._get_finite_data(selection)
 
-        if data.size == 0:
-            return {
-                "strict_positive_min": None,
-                "positive_min": None,
-                "min": None,
-                "max": None,
-                "mean": None,
-                "std": None,
-            }
+        return get_array_stats(data)
 
-        cast = float if np.issubdtype(data.dtype, np.floating) else int
-        strict_positive_data = data[data > 0]
-        positive_data = data[data >= 0]
-        return {
-            "strict_positive_min": cast(np.min(strict_positive_data))
-            if strict_positive_data.size != 0
-            else None,
-            "positive_min": cast(np.min(positive_data))
-            if positive_data.size != 0
-            else None,
-            "min": cast(np.min(data)),
-            "max": cast(np.max(data)),
-            "mean": cast(np.mean(data)),
-            "std": cast(np.std(data)),
-        }
-
-    def _get_finite_data(self, selection: Optional[str]) -> np.ndarray:
+    def _get_finite_data(self, selection: Selection) -> np.ndarray:
         data = np.array(self.data(selection), copy=False)  # So it works with scalars
 
         if not np.issubdtype(data.dtype, np.floating):
@@ -147,6 +185,7 @@ class GroupContent(ResolvedEntityContent[h5py.Group]):
     def __init__(self, path: str, h5py_entity: h5py.Group, h5file: h5py.File):
         super().__init__(path, h5py_entity)
         self._h5file = h5file
+        """ File in which the entity was resolved. This is needed to resolve child entity. """
 
     def _get_child_metadata_content(self, depth=0):
         return [
@@ -156,8 +195,13 @@ class GroupContent(ResolvedEntityContent[h5py.Group]):
             for child_path in self._h5py_entity.keys()
         ]
 
-    def metadata(self, depth=1):
-        if depth == 0:
+    def metadata(self, depth: int = 1):
+        """Metadata of the group. Recursively includes child metadata if depth > 0.
+
+        :parameter depth: The level of child metadata resolution.
+        :returns: {"attributes": AttributeMetadata, "children": ChildMetadata, "name": str, "type": str}
+        """
+        if depth <= 0:
             return super().metadata()
 
         return sorted_dict(
@@ -167,6 +211,17 @@ class GroupContent(ResolvedEntityContent[h5py.Group]):
 
 
 def create_content(h5file: h5py.File, path: Optional[str], resolve_links: bool = True):
+    """
+    Factory function to get entity content from a HDF5 file.
+    This handles external/soft link resolution and dataset decompression.
+
+    :param h5file: An open HDF5 file containing the entity
+    :param path: Path to the entity in the file.
+    :param resolve_links: Whether external and soft links should be resolved.
+    :raises h5grove.utils.PathError: If the path cannot be found in the file
+    :raises h5grove.utils.LinkError: If a link cannot be resolved when resolve_links is True
+    :raises TypeError: If encountering an unsupported h5py entity
+    """
     if path is None:
         path = "/"
 
